@@ -106,10 +106,6 @@ CBufr::CBufr(request* s) :
 
 CBufr::~CBufr()
 {
-    mars.debug = true;
-    print_all_requests(r);
-    mars.debug = false;
-
     const char* t = get_value(r, "TEMPORARY", 0);
     const char* p = get_value(r, "PATH", 0);
     if (t && p) {
@@ -356,7 +352,8 @@ CGeopts::CGeopts(CGeopts* p, fieldset* v, int n, bool nearest, bool nearestValid
         double near_lat = DBL_MAX;
         double near_lon = DBL_MAX;
         MvGeoP1 &pt = gpts[i];
-        if (!pt.latlon_missing()) {  // if lat/lon are missing, then it will be DBL_MAX
+        // if lat/lon are missing, then it will be DBL_MAX
+        if (!pt.latlon_missing() && pt.isLocationValid()) {  // if lat/lon are missing, then it will be DBL_MAX
             if (nearest) {
                 if (storeLocs) {
                     MvGridPoint gridp = fld->nearestGridpointInfo(pt.lon_x(), pt.lat_y(), nearestValid);
