@@ -1586,7 +1586,7 @@ void BufrFilterEngine::toGeopoints(const std::string& fName, const std::string&
 
             //station id
             if (keys[0])
-                out << keys[0]->valueAsString(i) << mySeparator;
+                out << metview::stationIdForWritingToFile(keys[0]->valueAsString(i)) << mySeparator;
             else
                 out << "?" << mySeparator;
 
